BeginPackage["HarmonicSums`"];

HarmonicSum::usage =
   "HarmonicSum[{a1,...,am},z] is the analytic continuation of the nested" \
   <> " harmonic sum Sum[HarmonicSum[{a2,...,am},z]/i^a1, {i,1,z}] to complex" \
   <> " values of z";

HarmonicSumCache::usage =
   "HarmonicSumCache[{a1,...,am},z] stores the numeric result for" \
   <> " HarmonicSum[{a1,...,am},z]. To prevent excessive memory usage use" \
   <> " Clear[HarmonicSumCache] regularly";

Begin["`Private`"];
HarmonicSum[{1},n_] := PolyGamma[0,n+1] + EulerGamma;
HarmonicSum[{k_},n_] := (-1)^(k+1) * PolyGamma[k-1,n+1]/(k-1)! + Zeta[k];

(* relations from shuffle algebra *)
HarmonicSum[{k_,k_},n_] := 1/2*(
   HarmonicSum[{k},n]^2 + HarmonicSum[{2*Abs[k]},n]
);

(* equations in Appendix A of arXiv:0901.3106  *)
HarmonicSum[{1,1,1},n_] := (
   + HarmonicSum[{1},n]^3 \
   + 3*HarmonicSum[{1},n]*HarmonicSum[{2},n] \
   + 2*HarmonicSum[{3},n]
)/6;

(* minimum argument for large N expansion *)
NCut = 20;
HarmonicSum[{k1_Integer,k__Integer},n_] := (
   HarmonicSumLargeN[{k1,k},n]
) /; Re[n] >= NCut;

(* eq. 24 of arXiv:0902.2148 *)
(* cache the result - the caller is responsible for clearing the cache *)
HarmonicSum[{k1_Integer,k__Integer},n_] := Module[
   {m,shift = 1 + NCut - Floor[Re[n]],res = HarmonicSumCache[{k1,k},n]},
   If[NumberQ[res], Return[res]];
   HarmonicSumCache[{k1,k},n] = (
      + HarmonicSumLargeN[{k1,k},n + shift] \
      - Sum[Sign[k1]^(m+n)/(m+n)^Abs[k1] * HarmonicSum[{k},m+n],{m,1,shift}]
   );
   Return[HarmonicSumCache[{k1,k},n]];
] /; Re[n] < NCut;

(* large N expansions (see arXiv:0902.2148) *)
HarmonicSumLargeN[{2,1},n_] = Collect[
   (Log[n]+EulerGamma)*(691/2730*n^(-13)-5/66*n^(-11)+1/30*n^(-9)-1/42*n^(-7)+1/30*n^(-5)-1/6*n^(-3)+1/2*n^(-2)-n^(-1))+54589/65520*n^(-14)-95538931/281080800*n^(-13)-7/33*n^(-12)+327053/3659040*n^(-11)+37/480*n^(-10)-247/7560*n^(-9)-11/252*n^(-8)+601/35280*n^(-7)+11/240*n^(-6)-1/100*n^(-5)-1/6*n^(-4)+13/36*n^(-3)-1/4*n^(-2)-n^(-1)+2*Zeta[3],
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{3,1},n_] = Collect[
(Log[n]+EulerGamma)*(691/420*n^(-14)-5/12*n^(-12)+3/20*n^(-10)-1/12*n^(-8)+1/12*n^(-6)-1/4*n^(-4)+1/2*n^(-3)-1/2*n^(-2))-134178917/60540480*n^(-14)-221/264*n^(-13)+65833/133056*n^(-12)+121/480*n^(-11)-209/1400*n^(-10)-19/168*n^(-9)+629/10080*n^(-8)+7/80*n^(-7)-23/720*n^(-6)-5/24*n^(-5)+17/48*n^(-4)-1/6*n^(-3)-1/4*n^(-2)+1/2*Zeta[2]^2,
   Log[n],
   N[HornerForm[#]] &
];


HarmonicSumLargeN[{3,2},n_] = N[HornerForm[
      42376/45045*n^(-13)-785/2772*n^(-11)+118/945*n^(-9)-37/420*n^(-7)+37/60*n^(-5)-1/2*Zeta[2]*n^(-2)+(-725/264+691/420*Zeta[2])*n^(-14)+(-5/8-1/4*Zeta[2])*n^(-4)+(-7/24+1/12*Zeta[2])*n^(-6)+(-11/42+3/20*Zeta[2])*n^(-10)+(13/80-1/12*Zeta[2])*n^(-8)+(1/3+1/2*Zeta[2])*n^(-3)+(169/240-5/12*Zeta[2])*n^(-12)+(-9/2*Zeta[5]+3*Zeta[2]*Zeta[3])
]];

HarmonicSumLargeN[{4,1},n_] = Collect[
   (Log[n]+EulerGamma)*(-5/3*n^(-13)+1/2*n^(-11)-2/9*n^(-9)+1/6*n^(-7)-1/3*n^(-5)+1/2*n^(-4)-1/3*n^(-3))-179/66*n^(-14)+557347/308880*n^(-13)+331/480*n^(-12)-3769/8316*n^(-11)-127/504*n^(-10)+173/1134*n^(-9)+3/20*n^(-8)-157/2520*n^(-7)-1/4*n^(-6)+7/20*n^(-5)-1/8*n^(-4)-1/9*n^(-3)+(3*Zeta[5]-Zeta[2]*Zeta[3]),
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{4,2},n_] = N[HornerForm[
      18329/4620*n^(-14)+5/3*n^(-13)-5063/5040*n^(-12)-43/84*n^(-11)+101/280*n^(-10)+1/4*n^(-9)-143/720*n^(-8)-1/3*n^(-7)+25/36*n^(-6)-3/5*n^(-5)+1/4*n^(-4)-5/3*Zeta[2]*n^(-13)+1/2*Zeta[2]*n^(-11)-2/9*Zeta[2]*n^(-9)+1/6*Zeta[2]*n^(-7)-1/3*Zeta[2]*n^(-5)+1/2*Zeta[2]*n^(-4)-1/3*Zeta[2]*n^(-3)-8/105*Zeta[2]^3+Zeta[3]^2
]];

HarmonicSumLargeN[{5,1},n_] = Collect[
   (Log[n]+EulerGamma)*(-65/12*n^(-14)+11/8*n^(-12)-1/2*n^(-10)+7/24*n^(-8)-5/12*n^(-6)+1/2*n^(-5)-1/4*n^(-4))+703757/133056*n^(-14)+793/480*n^(-13)-67751/60480*n^(-12)-253/504*n^(-11)+1037/3360*n^(-10)+19/80*n^(-9)-97/960*n^(-8)-7/24*n^(-7)+25/72*n^(-6)-1/10*n^(-5)-1/16*n^(-4)+2/5*Zeta[2]^3-1/2*Zeta[3]^2,
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{2,1,1},n_] = Collect[
   (Log[n]+EulerGamma)*(54589/65520*n^(-14)-95538931/281080800*n^(-13)-7/33*n^(-12)+327053/3659040*n^(-11)+37/480*n^(-10)-247/7560*n^(-9)-11/252*n^(-8)+601/35280*n^(-7)+11/240*n^(-6)-1/100*n^(-5)-1/6*n^(-4)+13/36*n^(-3)-1/4*n^(-2)-n^(-1))+(Log[n]+EulerGamma)^2*(691/5460*n^(-13)-5/132*n^(-11)+1/60*n^(-9)-1/84*n^(-7)+1/60*n^(-5)-1/12*n^(-3)+1/4*n^(-2)-1/2*n^(-1))-4458387211/3632428800*n^(-14)+3349/11880*n^(-12)-107491/1209600*n^(-10)+403/10080*n^(-8)-23/960*n^(-6)+37/96*n^(-4)+6/5*Zeta[2]^2+(-1-1/2*Zeta[2])*n^(-1)+(-79/216-1/12*Zeta[2])*n^(-3)+(-71825424101/204626822400+691/5460*Zeta[2])*n^(-13)+(-6587/36000+1/60*Zeta[2])*n^(-5)+(-120283/1814400+1/60*Zeta[2])*n^(-9)+(20231/329280-1/84*Zeta[2])*n^(-7)+(1/8+1/4*Zeta[2])*n^(-2)+(7459229/59628800-5/132*Zeta[2])*n^(-11),
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{2,2,1},n_] = Collect[
   (Log[n]+EulerGamma)*(-3015581/12612600*n^(-14)-115/132*n^(-13)+481/6930*n^(-12)+4/15*n^(-11)-121/4200*n^(-10)-31/252*n^(-9)+23/1260*n^(-8)+1/10*n^(-7)-7/360*n^(-6)-1/4*n^(-5)+13/24*n^(-4)-2/3*n^(-3)+1/2*n^(-2))-22725120853489/9090153072000*n^(-14)+991776839/1536796800*n^(-12)-2570231/10584000*n^(-10)+161009/1058400*n^(-8)-53/200*n^(-6)-11/288*n^(-4)-2*n^(-1)*Zeta[3]+2*Zeta[5]+(-17/36-1/3*Zeta[3])*n^(-3)+(-267/1540-5/33*Zeta[3])*n^(-11)+(-67/3150-1/21*Zeta[3])*n^(-7)+(7477/127008+1/15*Zeta[3])*n^(-9)+(61/180+1/15*Zeta[3])*n^(-5)+(68935/100672+691/1365*Zeta[3])*n^(-13)+(3/4+Zeta[3])*n^(-2),
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{3,1,1},n_] = Collect[
   (Log[n]+EulerGamma)*(-134178917/60540480*n^(-14)-221/264*n^(-13)+65833/133056*n^(-12)+121/480*n^(-11)-209/1400*n^(-10)-19/168*n^(-9)+629/10080*n^(-8)+7/80*n^(-7)-23/720*n^(-6)-5/24*n^(-5)+17/48*n^(-4)-1/6*n^(-3)-1/4*n^(-2))+(Log[n]+EulerGamma)^2*(691/840*n^(-14)-5/24*n^(-12)+3/40*n^(-10)-1/24*n^(-8)+1/24*n^(-6)-1/8*n^(-4)+1/4*n^(-3)-1/4*n^(-2))+7912873/5765760*n^(-13)-4896769/13305600*n^(-11)+5021/36288*n^(-9)-1481/20160*n^(-7)+101/240*n^(-5)+(-5243133011/10170800640+691/840*Zeta[2])*n^(-14)+(-65/192-1/8*Zeta[2])*n^(-4)+(-1813/8640+1/24*Zeta[2])*n^(-6)+(-1/8-1/4*Zeta[2])*n^(-2)+(-692203/6048000+3/40*Zeta[2])*n^(-10)+(743/8064-1/24*Zeta[2])*n^(-8)+(1/9+1/4*Zeta[2])*n^(-3)+(17432497/79833600-5/24*Zeta[2])*n^(-12)+(-1/2*Zeta[5]+Zeta[2]*Zeta[3]),
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{4,1,1},n_] = Collect[
   (Log[n]+EulerGamma)*(-179/66*n^(-14)+557347/308880*n^(-13)+331/480*n^(-12)-3769/8316*n^(-11)-127/504*n^(-10)+173/1134*n^(-9)+3/20*n^(-8)-157/2520*n^(-7)-1/4*n^(-6)+7/20*n^(-5)-1/8*n^(-4)-1/9*n^(-3))+(Log[n]+EulerGamma)^2*(-5/6*n^(-13)+1/4*n^(-11)-1/9*n^(-9)+1/12*n^(-7)-1/6*n^(-5)+1/4*n^(-4)-1/6*n^(-3))+171053/36960*n^(-14)+203482541/674593920*n^(-13)-47587/44800*n^(-12)-4438559/24393600*n^(-11)+337/1008*n^(-10)+27019/204120*n^(-9)-853/5760*n^(-8)-5569/23520*n^(-7)+11/24*n^(-6)-193/600*n^(-5)+3/32*n^(-4)-1/27*n^(-3)-5/6*Zeta[2]*n^(-13)+1/4*Zeta[2]*n^(-11)-1/9*Zeta[2]*n^(-9)+1/12*Zeta[2]*n^(-7)-1/6*Zeta[2]*n^(-5)+1/4*Zeta[2]*n^(-4)-1/6*Zeta[2]*n^(-3)+89/210*Zeta[2]^3-1/2*Zeta[3]^2,
   Log[n],
   N[HornerForm[#]] &
];

HarmonicSumLargeN[{2,1,1,1},n_] = Collect[
   (Log[n]+EulerGamma)*(-4458387211/3632428800*n^(-14)+3349/11880*n^(-12)-107491/1209600*n^(-10)+403/10080*n^(-8)-23/960*n^(-6)+37/96*n^(-4)+(-1-1/2*Zeta[2])*n^(-1)+(-79/216-1/12*Zeta[2])*n^(-3)+(-71825424101/204626822400+691/5460*Zeta[2])*n^(-13)+(-6587/36000+1/60*Zeta[2])*n^(-5)+(-120283/1814400+1/60*Zeta[2])*n^(-9)+(20231/329280-1/84*Zeta[2])*n^(-7)+(1/8+1/4*Zeta[2])*n^(-2)+(7459229/59628800-5/132*Zeta[2])*n^(-11))+(Log[n]+EulerGamma)^2*(54589/131040*n^(-14)-95538931/562161600*n^(-13)-7/66*n^(-12)+327053/7318080*n^(-11)+37/960*n^(-10)-247/15120*n^(-9)-11/504*n^(-8)+601/70560*n^(-7)+11/480*n^(-6)-1/200*n^(-5)-1/12*n^(-4)+13/72*n^(-3)-1/8*n^(-2)-1/2*n^(-1))+(Log[n]+EulerGamma)^3*(691/16380*n^(-13)-5/396*n^(-11)+1/180*n^(-9)-1/252*n^(-7)+1/180*n^(-5)-1/36*n^(-3)+1/12*n^(-2)-1/6*n^(-1))+4*Zeta[5]+(-1-1/2*Zeta[2]-1/3*Zeta[3])*n^(-1)+(-462772057751/1017080064000+54589/131040*Zeta[2])*n^(-14)+(-127/384-1/12*Zeta[2])*n^(-4)+(-679/3456+11/480*Zeta[2])*n^(-6)+(-5080026913/31877556480+327053/7318080*Zeta[2]-5/198*Zeta[3])*n^(-11)+(-135439/1512000+37/960*Zeta[2])*n^(-10)+(-3670669/124467840+601/70560*Zeta[2]-1/126*Zeta[3])*n^(-7)+(497429/8164800-247/15120*Zeta[2]+1/90*Zeta[3])*n^(-9)+(1/16-1/8*Zeta[2]+1/6*Zeta[3])*n^(-2)+(27527/362880-11/504*Zeta[2])*n^(-8)+(65/648+13/72*Zeta[2]-1/18*Zeta[3])*n^(-3)+(131887/760320-7/66*Zeta[2])*n^(-12)+(33919/90000-1/200*Zeta[2]+1/90*Zeta[3])*n^(-5)+(8616607154069/14963336388000-95538931/562161600*Zeta[2]+691/8190*Zeta[3])*n^(-13),
   Log[n],
   N[HornerForm[#]] &
];

End[];
EndPackage[];
